/****************************************************************************** 
StoneFlower & Anatoly Berezkin
March 2020

Development environment specifics:
Written in Arduino 1.6.0
*****************************************************************************/
#include <SPI.h>
#include <HighPowerStepperDriver.h>

//Declare pin functions on board
#define EN_PIN  2
#define DIR_PIN 4
#define STP_PIN 3
#define CS_PHD  9
#define CS_RAM  10
#define MOSI_PIN  11
#define MISO_PIN 12
#define SCK_PIN  13
#define push_btn 6 //if digital pin D6 is low, we rotate the motor (switch J6)

//Declare variables for functions
char user_input;
int x;
int y;
int state;
int microstepping;
int microstepping_old;
int feed_in;
int duration;
float spd;

HighPowerStepperDriver sd;

void setup() {
  Serial.begin(9600);
  while(!Serial);
  // Drive the STEP and DIR pins low initially.
  pinMode(STP_PIN, OUTPUT);
  digitalWrite(STP_PIN, LOW);
  pinMode(EN_PIN, OUTPUT);
  pinMode(DIR_PIN, OUTPUT);
  digitalWrite(DIR_PIN, LOW);
  pinMode(CS_PHD, OUTPUT);
  pinMode(CS_RAM, OUTPUT);
  pinMode(push_btn, INPUT);
  
  SPI.begin();
  //Settings of the RAM EXTRUDER
  sd.setChipSelectPin(CS_RAM);
  pinMode(MISO_PIN, INPUT_PULLUP);
  // Give the driver some time to power up.
  delay(1);
  // Reset the driver to its default settings and clear latched status
  // conditions.
  sd.resetSettings();
  sd.clearStatus();
  // Select auto mixed decay.  TI's DRV8711 documentation recommends this mode
  // for most applications, and we find that it usually works well.
  sd.setDecayMode(HPSDDecayMode::AutoMixed);
  // Set the current limit. You should change the number here to an appropriate
  // value for your particular system.
  sd.setCurrentMilliamps36v4(1600);
  // Set the number of microsteps that correspond to one full step.
  sd.setStepMode(HPSDStepMode::MicroStep4);
  // Enable the motor outputs.
  sd.enableDriver();

 //Settings of the PRINT HEAD
  sd.setChipSelectPin(CS_PHD);
  pinMode(MISO_PIN, INPUT_PULLUP);
  // Reset the driver to its default settings and clear latched status
  // conditions.
  sd.resetSettings();
  sd.clearStatus();
  // Select auto mixed decay.  TI's DRV8711 documentation recommends this mode
  // for most applications, and we find that it usually works well.
  sd.setDecayMode(HPSDDecayMode::AutoMixed);
  // Set the current limit. You should change the number here to an appropriate
  // value for your particular system.
  sd.setCurrentMilliamps36v4(1600);
  // Set the number of microsteps that correspond to one full step.
  sd.setStepMode(HPSDStepMode::MicroStep4);
  microstepping_old = 4;
  // Enable the motor outputs.
  sd.enableDriver();
  
  digitalWrite(STP_PIN, LOW);
  digitalWrite(EN_PIN, LOW);  
}

//Main loop
void loop() {

    state = digitalRead(push_btn); //Read user input and trigger appropriate function
    microstepping = int(analogRead(A6)/205);
    if (microstepping!=microstepping_old)
    {
     microstepping_old = microstepping;
     switch (microstepping) {
      case 0:// 1
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep1);
          sd.enableDriver();
          break;
      case 1:// 1/2
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep2);
          sd.enableDriver();
          break;
      case 2:// 1/4
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep4);
          sd.enableDriver();
          break;
      case 3:// 1/8
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep8);
          sd.enableDriver();
          break;
      case 4:// 1/16
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep16);
          sd.enableDriver();
          break;
      default:// 1/16
          sd.disableDriver();
          sd.setStepMode(HPSDStepMode::MicroStep16);
          sd.enableDriver();
     }
    }
    feed_in = 1024-analogRead(A7);
    spd = 4.9+feed_in/1024.0*195.0; //speed from voltage ml/h
    duration = 303.746*60.0/spd;   //ram S

    if (state == LOW & spd > 5.0)
      {
           StepDefault();
      }
}

//Reset Easy Driver pins to default states

//Default microstep mode function
void StepDefault()
{
  for(x= 1; x<100; x++)  //Loop the stepping
  {
    PORTD = PORTD | B00001000; //set STP_PIN=3 to HIGH, to trigger one step forward. Other pins are unaffected.
    delayMicroseconds(duration);
    PORTD = PORTD & B11110111; //set STP_PIN=3 to LOW, so it can be triggered again. Other pins are unaffected.
    delayMicroseconds(duration);
  }
}
